"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const reader_1 = require("./reader");
const analyzer_1 = require("./analyzer");
const Mustache = require("mustache");
const fs = require("fs");
const del = require("del");
const moment = require("moment");
class Reporter {
    constructor() {
        this.theme = 'white';
        this.folderToReadReport = `${process.cwd()}/`;
        this.folderToWriteReport = `${process.cwd()}/report/gherkin-detailer/`;
        this.folderToReadTemplates = `${__dirname}/../templates/`;
        this.gherkins = [];
        this.templates = {};
        this.templatesView = {};
        this.templatePartials = {};
        this.reader = new reader_1.Reader();
        this.analyzer = new analyzer_1.Analyzer();
        this.reportFeaturesFiles = this.reportFeaturesFiles.bind(this);
    }
    createGherkinsReport(config) {
        this.theme = config.theme;
        this.folderToReadReport = config.analysisFolder || this.folderToReadReport;
        this.folderToWriteReport = config.outputFolder || this.folderToWriteReport;
        if (this.folderToWriteReport.substr(-1) !== '/') {
            this.folderToWriteReport += '/';
        }
        this.setupReportFolder();
        this.reader.readFeatureFilesFromFolder(this.folderToReadReport, this.reportFeaturesFiles);
    }
    setupReportFolder() {
        if (fs.existsSync(this.folderToWriteReport)) {
            del.sync(this.folderToWriteReport);
        }
        fs.mkdirSync(this.folderToWriteReport, { recursive: true });
        fs.copyFileSync(`${this.folderToReadTemplates}${this.theme}.css`, `${this.folderToWriteReport}style.css`);
    }
    readAllGherkins(readFiles) {
        return __awaiter(this, void 0, void 0, function* () {
            let rowsFiles = [];
            yield Promise.all(readFiles
                .map((readFile, indexFile) => __awaiter(this, void 0, void 0, function* () {
                const contentFile = yield this.reader.readContentFeatureFile(readFile);
                const rowsFile = this.reader.getRowsFeatureFile(contentFile);
                const gherkinsFile = this.analyzer.getGherkins(rowsFile, indexFile);
                const dataFile = Object.assign(Object.assign({}, gherkinsFile), { file: readFile.replace(process.cwd(), '.') });
                rowsFiles.push(dataFile);
            })));
            let gherkinsTexts = rowsFiles.map((rowFile) => {
                return rowFile.files.filter((file) => file.id !== '').map((file) => file.text);
            });
            gherkinsTexts = [].concat(...gherkinsTexts);
            let gherkinsObjects = rowsFiles.map((rowFile) => {
                return rowFile.files.filter((file) => file.id !== '').map((file) => file);
            });
            gherkinsObjects = [].concat(...gherkinsObjects);
            const similarities = this.analyzer.getSimilarities(gherkinsTexts, gherkinsObjects);
            rowsFiles = rowsFiles.map((rowFile) => {
                rowFile.files = rowFile.files.map((file) => {
                    return this.addSimilarityInfo(file, similarities);
                });
                rowFile.features = rowFile.features.map((file) => {
                    return this.addSimilarityInfo(file, similarities);
                });
                rowFile.scenarios = rowFile.scenarios.map((file) => {
                    return this.addSimilarityInfo(file, similarities);
                });
                rowFile.states = rowFile.states.map((file) => {
                    return this.addSimilarityInfo(file, similarities);
                });
                rowFile.actions = rowFile.actions.map((file) => {
                    return this.addSimilarityInfo(file, similarities);
                });
                rowFile.outcomes = rowFile.outcomes.map((file) => {
                    return this.addSimilarityInfo(file, similarities);
                });
                return rowFile;
            });
            return rowsFiles;
        });
    }
    addSimilarityInfo(file, similarities) {
        if (similarities[file.id]) {
            file.class = similarities[file.id].class;
            file.similarities = similarities[file.id].similarities;
        }
        return file;
    }
    readAllTemplates() {
        return __awaiter(this, void 0, void 0, function* () {
            const templatesNames = ['meta', 'menu', 'footer', 'files', 'features', 'scenarios', 'states', 'actions', 'outcomes'];
            const templates = {};
            yield Promise.all(templatesNames
                .map((templateName) => __awaiter(this, void 0, void 0, function* () {
                const readTemplate = yield this.reader.readContentFeatureFile(`${this.folderToReadTemplates}${templateName}.mustache`);
                templates[templateName] = readTemplate;
                return {
                    [templateName]: readTemplate
                };
            })));
            return templates;
        });
    }
    prepareReports() {
        this.templatesView = {
            list: this.gherkins,
            date: moment().format('YYYY/MM/DD'),
            time: moment().format('HH:mm:ss')
        };
        this.templatePartials = {
            meta: this.templates.meta,
            footer: this.templates.footer,
            menu: this.templates.menu
        };
    }
    handleError(error) {
        if (error) {
            console.error(error);
            process.exit(1);
            return;
        }
        return;
    }
    writeFilesReport() {
        const reportFilesList = Mustache.render(this.templates.files, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}index.html`, reportFilesList, this.handleError);
    }
    writeFeaturesReport() {
        const reportFeaturesList = Mustache.render(this.templates.features, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}features.html`, reportFeaturesList, this.handleError);
    }
    writeScenariosReport() {
        const reportScenariosList = Mustache.render(this.templates.scenarios, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}scenarios.html`, reportScenariosList, this.handleError);
    }
    writeStatesReport() {
        const reportStatesList = Mustache.render(this.templates.states, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}states.html`, reportStatesList, this.handleError);
    }
    writeActionsReport() {
        const reportActionsList = Mustache.render(this.templates.actions, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}actions.html`, reportActionsList, this.handleError);
    }
    writeOutcomesReport() {
        const reportOutcomesList = Mustache.render(this.templates.outcomes, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}outcomes.html`, reportOutcomesList, this.handleError);
    }
    reportFeaturesFiles(readError, readFiles) {
        return __awaiter(this, void 0, void 0, function* () {
            this.handleError(readError);
            this.gherkins = yield this.readAllGherkins(readFiles);
            this.templates = yield this.readAllTemplates();
            this.prepareReports();
            this.writeFilesReport();
            this.writeFeaturesReport();
            this.writeScenariosReport();
            this.writeStatesReport();
            this.writeActionsReport();
            this.writeOutcomesReport();
        });
    }
}
exports.Reporter = Reporter;
